@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2018 The git2r contributors
+ *  Copyright (C) 2013-2019 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -21,6 +21,7 @@
 #include "git2r_arg.h"
 #include "git2r_blob.h"
 #include "git2r_commit.h"
+#include "git2r_deprecated.h"
 #include "git2r_error.h"
 #include "git2r_repository.h"
 #include "git2r_S3.h"
@@ -53,39 +54,39 @@ SEXP git2r_object_lookup(SEXP repo, SEXP sha)
     len = LENGTH(STRING_ELT(sha, 0));
     if (GIT_OID_HEXSZ == len) {
         git_oid_fromstr(&oid, CHAR(STRING_ELT(sha, 0)));
-        error = git_object_lookup(&object, repository, &oid, GIT_OBJ_ANY);
+        error = git_object_lookup(&object, repository, &oid, GIT2R_OBJECT_ANY);
         if (error)
             goto cleanup;
     } else {
         git_oid_fromstrn(&oid, CHAR(STRING_ELT(sha, 0)), len);
-        error = git_object_lookup_prefix(&object, repository, &oid, len, GIT_OBJ_ANY);
+        error = git_object_lookup_prefix(&object, repository, &oid, len, GIT2R_OBJECT_ANY);
         if (error)
             goto cleanup;
     }
 
     switch (git_object_type(object)) {
-    case GIT_OBJ_COMMIT:
+    case GIT2R_OBJECT_COMMIT:
         PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_commit));
         nprotect++;
         Rf_setAttrib(result, R_ClassSymbol,
                      Rf_mkString(git2r_S3_class__git_commit));
         git2r_commit_init((git_commit*)object, repo, result);
         break;
-    case GIT_OBJ_TREE:
+    case GIT2R_OBJECT_TREE:
         PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_tree));
         nprotect++;
         Rf_setAttrib(result, R_ClassSymbol,
                      Rf_mkString(git2r_S3_class__git_tree));
         git2r_tree_init((git_tree*)object, repo, result);
         break;
-    case GIT_OBJ_BLOB:
+    case GIT2R_OBJECT_BLOB:
         PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_blob));
         nprotect++;
         Rf_setAttrib(result, R_ClassSymbol,
                      Rf_mkString(git2r_S3_class__git_blob));
         git2r_blob_init((git_blob*)object, repo, result);
         break;
-    case GIT_OBJ_TAG:
+    case GIT2R_OBJECT_TAG:
         PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_tag));
         nprotect++;
         Rf_setAttrib(result, R_ClassSymbol,
@@ -104,7 +105,7 @@ cleanup:
         UNPROTECT(nprotect);
 
     if (error)
-        git2r_error(__func__, giterr_last(), NULL, NULL);
+        git2r_error(__func__, GIT2R_ERROR_LAST(), NULL, NULL);
 
     return result;
 }
